#include "CApplicationSocket.h"

CApplicationSocket::CApplicationSocket(QString name,int descriptor)
    : m_name(name), m_socket(descriptor)
{
    connect(&m_thread,SIGNAL(started()),SLOT(OnInitialize()));
    connect(this,SIGNAL(disconnected()),SLOT(OnDisconnect()));
    connect(this,SIGNAL(WriteMessage(QString)),SLOT(OnMessage(QString)));

    m_thread.start();
    moveToThread(&m_thread);
}

CApplicationSocket::~CApplicationSocket()
{
    if(m_thread.isRunning())
    {
        m_thread.quit();
        m_thread.wait();
    }

    qDebug(tr("Connection to %1 closed.").arg(m_name).toUtf8().data());
}

void CApplicationSocket::SendMessage(QString message)
{
    // Emit a signal that tells the socket thread
    // we have a message to send, and then send that signal.
    emit WriteMessage(message);
}

void CApplicationSocket::OnInitialize()
{
    qDebug(tr("Connection to %1 established.").arg(m_name).toUtf8().data());

    // Set the socket descriptor
    setSocketDescriptor(m_socket);

    // Now we can respond to requests and replies from
    // the application.
    connect(this,SIGNAL(readyRead()),SLOT(OnData()));
}

void CApplicationSocket::OnData()
{
    // read to the end of the next line
    QByteArray data = readLine().trimmed();

    // Split it into command / argument format
    QString command;
    QString arguments;

    if(data.indexOf(' ') != -1)
    {
        int iPos = data.indexOf(' ');

        command   = data.left(iPos);
        arguments = data.mid(iPos + 1);
    }
    else
        command = data;

    // Display the command
    qDebug(tr("%1: %2").arg(m_name).arg(command).toUtf8().data());

    if(command == "ONLINE")
        CommandONLINE(arguments);
    else if(command == "OFFLINE")
        CommandOFFLINE();

    // Otherwise, it's probably a custom message,
    // so send it to the ProcessMessage function
    ProcessMessage(command,arguments);
}

void CApplicationSocket::OnMessage(QString message)
{
    write(tr("%1\r\n").arg(message).toUtf8());
    waitForBytesWritten();
}

void CApplicationSocket::OnDisconnect()
{
    // We emit a signal indicating that
    // we are done and that we should be
    // deleted.

    emit Finished(this);
}

void CApplicationSocket::CommandONLINE(QString data)
{
    qDebug("Application online.");
}

void CApplicationSocket::CommandOFFLINE()
{
    qDebug("Going offline.");
}
